package com.huimv.admin.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.huimv.admin.common.token.TokenSign;
import com.huimv.admin.common.utils.DataUill;
import com.huimv.admin.common.utils.Result;
import com.huimv.admin.common.utils.ResultCode;
import com.huimv.admin.entity.*;
import com.huimv.admin.mapper.*;
import cn.hutool.core.date.DateTime;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.admin.common.token.TokenSign;
import com.huimv.admin.common.utils.*;
import com.huimv.admin.entity.*;
import com.huimv.admin.entity.vo.FactoryDirectorVo;
import com.huimv.admin.mapper.*;
import com.huimv.admin.service.IBillPersonnelAdmissionService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

import javax.servlet.http.HttpServletRequest;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * <p>
 * 入场申请 服务实现类
 * </p>
 *
 * @author author
 * @since 2023-07-21
 */
@Service
public class BillPersonnelAdmissionServiceImpl extends ServiceImpl<BillPersonnelAdmissionMapper, BillPersonnelAdmission> implements IBillPersonnelAdmissionService {
    @Resource
    private BillPersonnelAdmissionMapper billPersonnelAdmissionMapper;
    @Resource
    private BaseProcessMapper baseProcessMapper;
    @Resource
    private BillPcrMapper billPcrMapper;
    @Resource
    private BaseLocationMapper baseLocationMapper;
    @Resource
    private BillIsolateMapper billIsolateMapper;
    @Resource
    private BillCleanMapper billCleanMapper;
    @Resource
    private SysAccountMultilevelMapper sysAccountMultilevelMapper;
    @Autowired
    private UploadImage uploadImage;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result addAdmission(HttpServletRequest httpServletRequest, String admissionType, String vistitType, String userName, String phone, String workName, String papersType, String papersCode, MultipartFile imgUrl,String img, String vistitDate, String vistitReson, String destId, String destName,String farmId) throws IOException {
        Integer userId = TokenSign.getMemberIdByJwtToken(httpServletRequest);
        if (!PhoneNumberValidator.isValidPhoneNumber(phone)) {
            return new Result(10001,"手机号格式错误！",false);
        }
        if (papersType.equals("0")){
            if (!IDCardValidator.isValidIDCard(papersCode)){
                return new Result(10001,"身份证号格式错误!",false);
            }
        }
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm");
        BillPersonnelAdmission billPersonnelAdmission = new BillPersonnelAdmission();
        billPersonnelAdmission.setAdmissionType(Integer.parseInt(admissionType));
        billPersonnelAdmission.setVistitType(Integer.parseInt(vistitType));
        billPersonnelAdmission.setDestId(Integer.parseInt(destId));
        billPersonnelAdmission.setDestName(destName);
        billPersonnelAdmission.setAdmissionUserId(userId);
        billPersonnelAdmission.setAdmissionUserName(userName);
        billPersonnelAdmission.setVistitDate(LocalDateTime.parse(vistitDate,dateTimeFormatter));
        billPersonnelAdmission.setSubDate(LocalDateTime.now());
        billPersonnelAdmission.setPhone(phone);
        billPersonnelAdmission.setWorkName(workName);
        billPersonnelAdmission.setPapersType(Integer.parseInt(papersType));
        billPersonnelAdmission.setPapersCode(papersCode);
        if (ObjectUtil.isNotEmpty(imgUrl)){
            if (uploadImage.getImageCom(imgUrl).equals("上传失败")) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return new Result(10001,"图片上传失败",false);
            }
            billPersonnelAdmission.setImgUrl(uploadImage.getImageCom(imgUrl));
        }
        if (ObjectUtil.isNotEmpty(img)){
            billPersonnelAdmission.setImgUrl(img);
        }
        billPersonnelAdmission.setVistitReson(vistitReson);
        billPersonnelAdmission.setFarmId(Integer.parseInt(farmId));
        BaseProcess baseProcess = new BaseProcess();
        baseProcess.setDestName(destName);
        baseProcess.setDestId(Integer.parseInt(destId));
        baseProcess.setAdmissionUserId(userId);
        baseProcess.setAdmissionUserName(userName);
        baseProcess.setSubDate(DateTime.now());
        baseProcess.setUpdateDate(LocalDateTime.now().format(dateTimeFormatter));
        baseProcess.setCurrentStatus("0");
        //获取位置
        BaseLocation baseLocation = baseLocationMapper.selectOne(new QueryWrapper<BaseLocation>().eq("farm_id", farmId).eq("vistit_type", vistitType).eq("parent_id", 0));
        if (ObjectUtil.isEmpty(baseLocation)){
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return new Result(10001,"该来访类型位置不存在",false);
        }
        baseProcess.setCurrentLocation(baseLocation.getLocationName());
        baseProcess.setCurrentLocationId(baseLocation.getId().toString());
        baseProcess.setFarmId(Integer.parseInt(farmId));
        baseProcessMapper.insert(baseProcess);
        billPersonnelAdmission.setProcessId(baseProcess.getId());
        billPersonnelAdmissionMapper.insert(billPersonnelAdmission);
        return Result.SUCCESS();
    }

    @Override
    public Result listPersonalAdmission(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String pageNo = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        Page<BillPersonnelAdmission> page = new Page<>(Integer.parseInt(pageNo),Integer.parseInt(pageSize));
        Page<BillPersonnelAdmission> billPersonnelAdmissionPage = billPersonnelAdmissionMapper.selectPage(page, new QueryWrapper<BillPersonnelAdmission>().eq("admission_user_id", TokenSign.getMemberIdByJwtToken(httpServletRequest)).orderByDesc("sub_date"));
        return new Result(ResultCode.SUCCESS,billPersonnelAdmissionPage);
    }

    @Override
    public Result listAdmission(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String pageNo = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        String type = paramsMap.get("type");
        String farmId = paramsMap.get("farmId");
        QueryWrapper<BillPersonnelAdmission> queryWrapper = new QueryWrapper<>();
        Integer userType = TokenSign.getUserTypeByJwtToken(httpServletRequest);
        if (userType == 3){
            //生物安全负责人
            queryWrapper.eq("admission_type",0);
        }else if (userType == 2){
            //紧急入场
            queryWrapper.ne("admission_type",0);
        }
        switch (type){
            case "1":
                queryWrapper.eq("bill_status",0);
                break;
            case "2":
                queryWrapper.eq("bill_status",1);
                break;
            case "3":
                queryWrapper.eq("bill_status",2);
                break;
            default: break;
        }
        queryWrapper.orderByDesc("sub_date").eq("farm_id",farmId);
//        List<BaseLocation> baseLocations = baseLocationMapper.selectList(new QueryWrapper<BaseLocation>().eq("serial_number", 0));
//        List<Integer> locationIds = new ArrayList<>();
//        for (BaseLocation baseLocation : baseLocations) {
//            locationIds.add(baseLocation.getId());
//        }
//        if (!locationIds.contains(Integer.parseInt(TokenSign.getWorkIdByJwtToken(httpServletRequest)))){
//            queryWrapper.eq("id",0);
//        }
        Page<BillPersonnelAdmission> page = new Page<>(Integer.parseInt(pageNo),Integer.parseInt(pageSize));
        Page<BillPersonnelAdmission> billPersonnelAdmissionPage = billPersonnelAdmissionMapper.selectPage(page,queryWrapper);
        return new Result(ResultCode.SUCCESS,billPersonnelAdmissionPage);
    }

    @Override
    @Transactional
    public Result handleAdmission(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String type = paramsMap.get("type");
        String id = paramsMap.get("id");
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm");
        BillPersonnelAdmission billPersonnelAdmission = billPersonnelAdmissionMapper.selectById(id);
        Integer userType = TokenSign.getUserTypeByJwtToken(httpServletRequest);
        billPersonnelAdmission.setPassDate(LocalDateTime.now());
        billPersonnelAdmission.setPassUserId(TokenSign.getMemberIdByJwtToken(httpServletRequest));
        billPersonnelAdmission.setPassUserName(TokenSign.getUserNameByJwtToken(httpServletRequest));
        BaseProcess baseProcess = baseProcessMapper.selectById(billPersonnelAdmission.getProcessId());
        List<BaseLocation> baseLocations = baseLocationMapper.selectList(new QueryWrapper<BaseLocation>().eq("serial_number", 0));
        List<Integer> locationIds = new ArrayList<>();
        for (BaseLocation baseLocation : baseLocations) {
            locationIds.add(baseLocation.getId());
        }
        if (StringUtils.isBlank(baseProcess.getUpdateDate())){
            baseProcess.setUpdateDate(LocalDateTime.now().format(dateTimeFormatter));
        }else {
            baseProcess.setUpdateDate(baseProcess.getUpdateDate()+","+LocalDateTime.now().format(dateTimeFormatter));
        }
        if (type.equals("0")){
            //拒绝
            billPersonnelAdmission.setBillStatus(2);
            baseProcess.setProcessType(2);
        }else {
            //通过
            billPersonnelAdmission.setBillStatus(1);
            //获取pcr位置
            String laseLocationId = baseProcess.getCurrentLocationId();
            BaseLocation laseLocation = baseLocationMapper.selectById(laseLocationId);
            BaseLocation baseLocation = baseLocationMapper.selectById(laseLocation.getNextId());
            if (userType == 3){
                if (locationIds.contains(billPersonnelAdmission.getDestId())){
                    baseProcess.setCurrentStatus("1");
                    baseProcess.setProcessType(1);
                }else {
                    baseProcess.setCurrentStatus("1,0");
                    baseProcess.setCurrentLocation(baseProcess.getCurrentLocation()+","+baseLocation.getLocationName());
                    baseProcess.setCurrentLocationId(baseProcess.getCurrentLocationId() +","+baseLocation.getId());
                    BillPcr billPcr = new BillPcr();
                    billPcr.setVistitType(billPersonnelAdmission.getVistitType());
                    billPcr.setDestName(billPersonnelAdmission.getDestName());
                    billPcr.setDestId(billPersonnelAdmission.getDestId());
                    billPcr.setPhone(billPersonnelAdmission.getPhone());
                    billPcr.setAdmissionUserId(billPersonnelAdmission.getAdmissionUserId());
                    billPcr.setAdmissionUserName(billPersonnelAdmission.getAdmissionUserName());
                    billPcr.setSubDate(LocalDateTime.now());
                    billPcr.setVistitDate(billPersonnelAdmission.getVistitDate());
                    billPcr.setFarmId(billPersonnelAdmission.getFarmId());
                    billPcr.setProcessId(billPersonnelAdmission.getProcessId());
                    billPcr.setAdmissionId(Integer.parseInt(id));
                    billPcr.setTestLocation(baseLocation.getLocationName());
                    billPcr.setTestLocationId(baseLocation.getId());
                    billPcrMapper.insert(billPcr);
                }
            }else {
                baseProcess.setCurrentStatus("1");
//                baseProcess.setCurrentLocation(baseProcess.getCurrentLocation()+","+baseLocation.getLocationName());
//                baseProcess.setCurrentLocationId(baseProcess.getCurrentLocationId() +","+baseLocation.getId());
                //TODO 紧急入场通过先不处理
            }
        }
        baseProcessMapper.updateById(baseProcess);
        billPersonnelAdmissionMapper.updateById(billPersonnelAdmission);
        return Result.SUCCESS();
    }

    @Override
    public Result listAll(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String type = paramsMap.get("type");
        String farmId = paramsMap.get("farmId");
        String pageNo = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        Page<FactoryDirectorVo> page = new Page<>(Integer.parseInt(pageNo),Integer.parseInt(pageSize));
        IPage<FactoryDirectorVo> factoryDirectorList = billPersonnelAdmissionMapper.listAll(type,farmId,page);
        return new Result(ResultCode.SUCCESS,factoryDirectorList);
    }

    @Override
    public Result getDetail(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String type = paramsMap.get("type");
        String id = paramsMap.get("id");
        if (type.equals("0")){
            return new Result(ResultCode.SUCCESS,billPersonnelAdmissionMapper.selectById(id));
        }else if (type.equals("1")){
            return new Result(ResultCode.SUCCESS,billPcrMapper.selectById(id));
        }else if (type.equals("2")){
            return new Result(ResultCode.SUCCESS,billCleanMapper.selectById(id));
        }else {
            return new Result(ResultCode.SUCCESS,billIsolateMapper.selectById(id));
        }
    }

    @Autowired
    private BillPcrMapper pcrMapper;
    @Autowired
    private BaseProcessMapper processMapper;
    @Autowired
    private BaseLocationMapper locationMapper;
    @Autowired
    private BillCleanMapper cleanMapper;
    @Autowired
    private BillIsolateMapper isolateMapper;

    @Override
    public Result listPrc(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String type = paramsMap.get("type");//类型
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if ("".equals(pageNum) || null == pageNum) {
            pageNum = "1";
        }
        if ("".equals(pageSize) || null == pageSize) {
            pageSize = "20";
        }
        Page<BillPcr> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        if ("".equals(type) || null == type) {
            type = "0";
        }
        String farmId = paramsMap.get("farmId");
        QueryWrapper<BillPcr> queryWrapper = new QueryWrapper<>();

        queryWrapper.eq("farm_id", farmId).orderByDesc("sub_date");
        if ("0".equals(type)) {
            queryWrapper.in("pcr_type",  1, 2, 3);
        }
        if ("1".equals(type)) {
            queryWrapper.eq("pcr_type", 1);
        }
        if ("2".equals(type)) {
            queryWrapper.eq("pcr_type", 2);
        }
        if ("3".equals(type)) {
            queryWrapper.eq("pcr_type",  3);
        }
        return new Result(ResultCode.SUCCESS,  pcrMapper.selectPage(page,queryWrapper));
    }

    @Override
    public Result listById(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        return new Result(ResultCode.SUCCESS, pcrMapper.selectById(id));
    }

    @Override
    public Result editStatus(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) throws ParseException {
        String farmId = paramsMap.get("farmId");
        String type = paramsMap.get("type");
        String id = paramsMap.get("id");
        String data = paramsMap.get("data");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        if (StringUtils.isBlank(data)) {
            data = sdf.format(new Date());
        }
        BillPcr billPcr = pcrMapper.selectById(id);

        QueryWrapper<BaseProcess> processQueryWrapper = new QueryWrapper<>();
        processQueryWrapper.eq("farm_id", farmId).eq("id", billPcr.getProcessId());
        BaseProcess baseProcess = processMapper.selectOne(processQueryWrapper);//进程

        QueryWrapper<BaseLocation> locationQueryWrapper = new QueryWrapper<>();
        locationQueryWrapper.eq("farm_id", farmId).eq("id", billPcr.getTestLocationId());
        BaseLocation baseLocation = locationMapper.selectOne(locationQueryWrapper);//当前pcr检测所在的位置


        if (baseProcess.getProcessType() == 2 || baseProcess.getProcessType() == 1) {
            return new Result(10001, "修改失败！该申请已提前结束或者被拒绝", false);
        } else {
            if (billPcr.getDestId() > billPcr.getTestLocationId()) { //说明当前不是目的地
                if ("1".equals(type)) {
                    billPcr.setBillStatus(1);
                    billPcr.setPcrType(2);
                    billPcr.setPassUserName(TokenSign.getUserNameByJwtToken(httpServletRequest));
                    billPcr.setPassUserId(TokenSign.getMemberIdByJwtToken(httpServletRequest));
                    billPcr.setTestLocation(baseLocation.getLocationName());
                    billPcr.setTestLocationId(baseLocation.getId());
                    billPcr.setPassDate(sdf.parse(sdf.format(new Date())));
                    billPcr.setCheckDate(sdf.parse(data));

                    //通过生成洗消记录
                    BillClean billClean = new BillClean();
                    billClean.setPhone(billPcr.getPhone());
                    billClean.setVistitType(billPcr.getVistitType());
                    billClean.setDestName(billPcr.getDestName());
                    billClean.setDestId(billPcr.getDestId());
                    billClean.setAdmissionUserName(billPcr.getAdmissionUserName());
                    billClean.setAdmissionUserId(billPcr.getAdmissionUserId());
                    billClean.setVistitDate(billPcr.getVistitDate());
                    billClean.setSubDate(billPcr.getSubDate());
                    billClean.setFarmId(billPcr.getFarmId());
                    billClean.setPassUserName(billPcr.getPassUserName());
                    billClean.setPassDate(billPcr.getPassDate());
                    billClean.setPassUserId(billPcr.getPassUserId());
                    billClean.setProcessId(billPcr.getProcessId());
                    billClean.setTestLocation(baseLocation.getNextLocation());
                    billClean.setTestLocationId(baseLocation.getNextId());
                    cleanMapper.insert(billClean);

                    baseProcess.setUpdateDate(baseProcess.getUpdateDate() + "," +sdf.format(new Date()));
                    baseProcess.setCurrentLocation(baseProcess.getCurrentLocation() + "," + baseLocation.getNextLocation());
                    baseProcess.setCurrentLocationId(baseProcess.getCurrentLocationId() + "," + baseLocation.getNextId());
                    String status = baseProcess.getCurrentStatus();
                    String substring = status.substring(0, status.length() - 1);

                    baseProcess.setCurrentStatus(substring + "1," + 0);

                    processMapper.updateById(baseProcess);
                    pcrMapper.updateById(billPcr);
                    return new Result(10000, "修改成功！", true);
                } else {
                    billPcr.setBillStatus(2);
                    billPcr.setPcrType(3);
                    billPcr.setPassUserName(TokenSign.getUserNameByJwtToken(httpServletRequest));
                    billPcr.setPassUserId(TokenSign.getMemberIdByJwtToken(httpServletRequest));
                    billPcr.setTestLocation(baseLocation.getLocationName());
                    billPcr.setTestLocationId(baseLocation.getId());
                    billPcr.setPassDate(sdf.parse(sdf.format(new Date())));
                    billPcr.setCheckDate(sdf.parse(data));

                    baseProcess.setUpdateDate(baseProcess.getUpdateDate() + "," +sdf.format(new Date()));
                    baseProcess.setCurrentLocation(baseProcess.getCurrentLocation() );
                    baseProcess.setCurrentLocationId(baseProcess.getCurrentLocationId());
                    String status = baseProcess.getCurrentStatus();
                    String substring = status.substring(0, status.length() - 1);

                    baseProcess.setCurrentStatus(substring + "2");
                    baseProcess.setProcessType(2);
                    processMapper.updateById(baseProcess);
                    pcrMapper.updateById(billPcr);
                    return new Result(10000, "修改成功！", true);
                }
            } else {
                if ("1".equals(type)) {
                    billPcr.setBillStatus(1);
                    billPcr.setPcrType(2);
                    billPcr.setPassUserName(TokenSign.getUserNameByJwtToken(httpServletRequest));
                    billPcr.setPassUserId(TokenSign.getMemberIdByJwtToken(httpServletRequest));
                    billPcr.setTestLocation(baseLocation.getLocationName());
                    billPcr.setTestLocationId(baseLocation.getId());
                    billPcr.setPassDate(sdf.parse(sdf.format(new Date())));
                    billPcr.setCheckDate(sdf.parse(data));

                    baseProcess.setUpdateDate(baseProcess.getUpdateDate() + "," +sdf.format(new Date()));
                    baseProcess.setCurrentLocation(baseProcess.getCurrentLocation());
                    baseProcess.setCurrentLocationId(baseProcess.getCurrentLocationId());
                    String status = baseProcess.getCurrentStatus();
                    String substring = status.substring(0, status.length() - 1);
                    baseProcess.setCurrentStatus(substring + "1" );
                    baseProcess.setProcessType(1);
                    processMapper.updateById(baseProcess);
                    pcrMapper.updateById(billPcr);
                    return new Result(10000, "修改成功！", true);
                } else {
                    baseProcess.setUpdateDate(baseProcess.getUpdateDate() + "," +sdf.format(new Date()));
                    baseProcess.setCurrentLocation(baseProcess.getCurrentLocation() );
                    baseProcess.setCurrentLocationId(baseProcess.getCurrentLocationId() );
                    String status = baseProcess.getCurrentStatus();
                    String substring = status.substring(0, status.length() - 1);
                    baseProcess.setCurrentStatus(substring + "2");
                    baseProcess.setProcessType(2);
                    billPcr.setBillStatus(2);
                    billPcr.setPcrType(3);
                    billPcr.setPassUserName(TokenSign.getUserNameByJwtToken(httpServletRequest));
                    billPcr.setPassUserId(TokenSign.getMemberIdByJwtToken(httpServletRequest));
                    billPcr.setTestLocation(baseLocation.getLocationName());
                    billPcr.setTestLocationId(baseLocation.getId());
                    billPcr.setPassDate(sdf.parse(sdf.format(new Date())));
                    billPcr.setCheckDate(sdf.parse(data));

                    processMapper.updateById(baseProcess);
                    pcrMapper.updateById(billPcr);
                    return new Result(10000, "修改成功！", true);
                }

            }
        }
    }

    @Override
    public Result listIsolate(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String type = paramsMap.get("type");//类型
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if ("".equals(pageNum) || null == pageNum) {
            pageNum = "1";
        }
        if ("".equals(pageSize) || null == pageSize) {
            pageSize = "20";
        }
        Page<BillIsolate> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        if ("".equals(type) || null == type) {
            type = "0";
        }
        String farmId = paramsMap.get("farmId");
        QueryWrapper<BillIsolate> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).orderByDesc("sub_date");
        if ("0".equals(type)) {
            queryWrapper.in("bill_status", 0, 1, 2, 3);
        }
        if ("1".equals(type)) {
            queryWrapper.eq("bill_status", 0);
        }
        if ("2".equals(type)) {
            queryWrapper.eq("bill_status",  1);
        }
        if ("3".equals(type)) {
            queryWrapper.in("bill_status", 2, 3);
        }
        return new Result(ResultCode.SUCCESS, isolateMapper.selectPage(page,queryWrapper));
    }

    @Override
    public Result isolateById(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        return new Result(ResultCode.SUCCESS, isolateMapper.selectById(id));
    }

    @Override
    public Result commitIsolate(HttpServletRequest httpServletRequest, BillIsolate billIsolate) {
        BillIsolate isolate = isolateMapper.selectById(billIsolate.getId());
        Date startDate = billIsolate.getIsolateStartDate();
        Integer dayNum = billIsolate.getIsolateDayNum();//隔离天数
        Date endDate = DataUill.getNextDay(startDate, dayNum);
        isolate.setIsolateStartDate(startDate);
        isolate.setIsolateEndDate(endDate);
        isolate.setIsolateDayNum(dayNum);
        isolate.setBillStatus(1);
        ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(10);
        Runnable task = new Runnable() {
            @Override
            public void run() {
                //这里写业务
                isolate.setBillStatus(2);
                isolateMapper.updateById(isolate);
            }
        };
        // 设定延迟时间（两小时后执行，单位为秒）
        long delay = 60 * 60 * dayNum;
        // 执行定时任务
        scheduler.schedule(task, delay, TimeUnit.SECONDS);
        isolateMapper.updateById(isolate);
        return new Result(10000, "提交成功！", true);
    }

    @Override
    public Result editIsolate(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) throws ParseException {
        String id = paramsMap.get("id");
        String farmId = paramsMap.get("farmId");
        String remark = paramsMap.get("remark");//是否提前结束隔离的标识
        BillIsolate billIsolate = isolateMapper.selectById(id);
        BaseProcess baseProcess = processMapper.selectById(billIsolate.getProcessId());
        QueryWrapper<BaseLocation> locationQueryWrapper = new QueryWrapper<>();
        locationQueryWrapper.eq(StringUtils.isNotBlank(farmId),"farm_id", farmId).eq("id", billIsolate.getIsolateLocationId());
        BaseLocation baseLocation = locationMapper.selectOne(locationQueryWrapper);//当前隔离所在的位置

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String end = sdf.format(billIsolate.getIsolateEndDate());
        String start = sdf.format(billIsolate.getIsolateStartDate());
        String now = sdf.format(new Date());
        int res = now.compareTo(end);
        if (res > 0) {//当前时间晚于隔离结束时间
            if (baseProcess.getDestId().equals(billIsolate.getIsolateLocationId())|| baseProcess.getDestId() >= 13) {
                //目的地就是当前隔离地点
                baseProcess.setProcessType(1);
                baseProcess.setCurrentLocation(baseProcess.getCurrentLocation());
                baseProcess.setCurrentLocationId(baseProcess.getCurrentLocationId());
            } else {
                baseProcess.setProcessType(0);
                baseProcess.setCurrentLocation(baseProcess.getCurrentLocation() + "," + baseLocation.getNextLocation());
                baseProcess.setCurrentLocationId(baseProcess.getCurrentLocationId() + "," + baseLocation.getNextId());
            }
            if (billIsolate.getIsolateLocationId() == 8) {
                //通过生成洗消记录
                BillClean billClean = new BillClean();
                billClean.setVistitType(billIsolate.getVistitType());
                billClean.setDestName(billIsolate.getDestName());
                billClean.setDestId(billIsolate.getDestId());
                billClean.setAdmissionUserName(billIsolate.getAdmissionUserName());
                billClean.setAdmissionUserId(billIsolate.getAdmissionUserId());
                billClean.setVistitDate(billIsolate.getVistitDate());
                billClean.setSubDate(billIsolate.getSubDate());
                billClean.setFarmId(billIsolate.getFarmId());
                billClean.setPassUserName(billIsolate.getPassUserName());
                billClean.setPassDate(billIsolate.getPassDate());
                billClean.setPassUserId(billIsolate.getPassUserId());
                billClean.setProcessId(billIsolate.getProcessId());
                billClean.setTestLocation("场内洗澡间1");
                billClean.setTestLocationId(10);
                billClean.setPhone(billIsolate.getPhone());
                cleanMapper.insert(billClean);
            }

            billIsolate.setIsolateRealEndDate(new Date());
            SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd");
            Calendar cal1 = Calendar.getInstance();
            Calendar cal2 = Calendar.getInstance();
            Date startDate = sdf1.parse(start);
            Date endDate = sdf1.parse(now);
            cal1.setTime(startDate);
            cal2.setTime(endDate);
            long days = (cal2.getTimeInMillis() - cal1.getTimeInMillis()) / (1000 * 3600 * 24);
            billIsolate.setIsolateRealDayNum((int) days);
            billIsolate.setBillStatus(2);
            billIsolate.setPassUserName(TokenSign.getUserNameByJwtToken(httpServletRequest));
            billIsolate.setPassUserId(TokenSign.getMemberIdByJwtToken(httpServletRequest));

            baseProcess.setUpdateDate(baseProcess.getUpdateDate() + "," + now);

            String status = baseProcess.getCurrentStatus();
            String substring = status.substring(0, status.length() - 1);
            baseProcess.setCurrentStatus(substring + "1," + 0);

            isolateMapper.updateById(billIsolate);
            processMapper.updateById(baseProcess);
            return new Result(10000, "正常解除隔离成功！", true);
        } else {
            //提前结束隔离
            if (baseProcess.getDestId().equals(billIsolate.getIsolateLocationId())) {
                //目的地就是当前隔离地点
                baseProcess.setProcessType(1);
                baseProcess.setCurrentLocation(baseProcess.getCurrentLocation());
                baseProcess.setCurrentLocationId(baseProcess.getCurrentLocationId());
            } else {
                baseProcess.setProcessType(0);
                baseProcess.setCurrentLocation(baseProcess.getCurrentLocation() + "," + baseLocation.getNextLocation());
                baseProcess.setCurrentLocationId(baseProcess.getCurrentLocationId() + "," + baseLocation.getNextId());
            }

            if (billIsolate.getIsolateLocationId() == 8) {
                //通过生成洗消记录
                BillClean billClean = new BillClean();
                billClean.setVistitType(billIsolate.getVistitType());
                billClean.setDestName(billIsolate.getDestName());
                billClean.setDestId(billIsolate.getDestId());
                billClean.setAdmissionUserName(billIsolate.getAdmissionUserName());
                billClean.setAdmissionUserId(billIsolate.getAdmissionUserId());
                billClean.setVistitDate(billIsolate.getVistitDate());
                billClean.setSubDate(billIsolate.getSubDate());
                billClean.setFarmId(billIsolate.getFarmId());
                billClean.setPassUserName(billIsolate.getPassUserName());
                billClean.setPassDate(billIsolate.getPassDate());
                billClean.setPassUserId(billIsolate.getPassUserId());
                billClean.setProcessId(billIsolate.getProcessId());
                billClean.setTestLocation("场内洗澡间1");
                billClean.setTestLocationId(10);
                billClean.setPhone(billIsolate.getPhone());
                cleanMapper.insert(billClean);
            }
            billIsolate.setIsolateRealEndDate(new Date());
            SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd");
            Calendar cal1 = Calendar.getInstance();
            Calendar cal2 = Calendar.getInstance();
            Date startDate = sdf1.parse(start);
            Date endDate = sdf1.parse(now);
            cal1.setTime(startDate);
            cal2.setTime(endDate);
            long days = (cal2.getTimeInMillis() - cal1.getTimeInMillis()) / (1000 * 3600 * 24);
            billIsolate.setIsolateRealDayNum((int) days);
            billIsolate.setBillStatus(3);
            billIsolate.setPassUserName(TokenSign.getUserNameByJwtToken(httpServletRequest));
            billIsolate.setPassUserId(TokenSign.getMemberIdByJwtToken(httpServletRequest));

            baseProcess.setUpdateDate(baseProcess.getUpdateDate() + "," + now);

            String status = baseProcess.getCurrentStatus();
            String substring = status.substring(0, status.length() - 1);
            baseProcess.setCurrentStatus(substring + "1," + 0);

            isolateMapper.updateById(billIsolate);
            processMapper.updateById(baseProcess);
            return new Result(10000, "提前解除隔离成功！", true);
        }
    }

    @Override
    public Result listClean(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String type = paramsMap.get("type");//类型
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if ("".equals(pageNum) || null == pageNum) {
            pageNum = "1";
        }
        if ("".equals(pageSize) || null == pageSize) {
            pageSize = "20";
        }
        Page<BillClean> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        if ("".equals(type) || null == type) {
            type = "0";
        }
        String farmId = paramsMap.get("farmId");
        QueryWrapper<BillClean> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).orderByDesc("sub_date");
        if ("0".equals(type)) {
            queryWrapper.in("bill_status", 0, 1, 2);
        }
        if ("1".equals(type)) {
            queryWrapper.eq("bill_status", 0);
        }
        if ("2".equals(type)) {
            queryWrapper.eq("bill_status", 1);
        }
        if ("3".equals(type)) {
            queryWrapper.eq("bill_status", 2);
        }
        return new Result(ResultCode.SUCCESS,  cleanMapper.selectPage(page,queryWrapper));

    }
}
