package com.huimv.admin.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.admin.common.utils.DataUill;
import com.huimv.admin.common.utils.Result;
import com.huimv.admin.common.utils.ResultCode;
import com.huimv.admin.entity.BasePigpen;
import com.huimv.admin.entity.EnergyData;
import com.huimv.admin.entity.EnergyElectricity;
import com.huimv.admin.entity.vo.EnergyElectricityDataVo;
import com.huimv.admin.mapper.BasePigpenMapper;
import com.huimv.admin.mapper.EnergyDataMapper;
import com.huimv.admin.mapper.EnergyElectricityMapper;
import com.huimv.admin.service.IEnergyElectricityService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.text.DecimalFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author author
 * @since 2023-02-14
 */
@Service
public class EnergyElectricityServiceImpl extends ServiceImpl<EnergyElectricityMapper, EnergyElectricity> implements IEnergyElectricityService {

    @Autowired
    private EnergyElectricityMapper energyElectricityMapper;
    @Autowired
    private BasePigpenMapper basePigpenMapper;
    @Autowired
    private EnergyDataMapper dataMapper;

    @Override
    public Result listPigpen(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String id = paramsMap.get("id");//层id

        JSONArray jsonArray = new JSONArray();
        QueryWrapper<BasePigpen> basePigpenQueryWrapper = new QueryWrapper<>();
        basePigpenQueryWrapper.eq("farm_id", farmId).like("other2", id);
        List<BasePigpen> basePigpens = basePigpenMapper.selectList(basePigpenQueryWrapper);//所有的单元
        for (BasePigpen basePigpen : basePigpens) {
            JSONObject jsonObject = new JSONObject();
            QueryWrapper<EnergyData> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("farm_id", farmId).eq("unit_id", basePigpen.getId());
            queryWrapper.select(" IFNULL(SUM(electricity_value),'0') electricityValue");
            DateTime dateTime = DateUtil.beginOfDay(new Date());
            queryWrapper.ge("create_date", dateTime);

            EnergyData energyElectricity = dataMapper.selectOne(queryWrapper);
            if (ObjectUtil.isEmpty(energyElectricity)) {
                jsonObject.put("buildName", basePigpen.getBuildName());
                jsonObject.put("value", 0);
                jsonObject.put("id", basePigpen.getId());
            } else {
                DecimalFormat df = new DecimalFormat("0.00");
                jsonObject.put("buildName", basePigpen.getBuildName());
                jsonObject.put("value", df.format(Float.parseFloat(energyElectricity.getElectricityValue())));
                jsonObject.put("id", basePigpen.getId());
            }
            jsonArray.add(jsonObject);
        }


        return new Result(ResultCode.SUCCESS, jsonArray);
    }

    @Override
    public Result listData(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String type = paramsMap.get("type");
        String id = paramsMap.get("id");
        String startTime = paramsMap.get("startTime");
        String endTime = paramsMap.get("endTime");

        if (type == null || type == "") {
            type = "3";
        }

        QueryWrapper<EnergyData> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("unit_id", id);
        List<EnergyElectricityDataVo> electricityList = null;
        //自定义查询
        if ("4".equals(type)) {
            startTime = startTime + " 00:00:00";
            endTime = endTime + " 23:59:59";
            queryWrapper.between("create_date", startTime, endTime);
            electricityList = dataMapper.listDay4(queryWrapper);
        }
        //本年
        else if ("3".equals(type)) {
            DateTime dateTime = DateUtil.beginOfYear(new Date());
            queryWrapper.ge("create_date", dateTime);
            List<EnergyElectricityDataVo> energyElectricityDataVos = dataMapper.listYear4(queryWrapper);
            electricityList = energyElectricityDataVos;
        }
        //本月
        else if ("2".equals(type)) {
            Date timesMonthmorning = DataUill.getTimesMonthmorning();
            queryWrapper.ge("create_date", timesMonthmorning);
            electricityList = dataMapper.listDay4(queryWrapper);
        }
        //本周
        else if ("1".equals(type)) {
            Calendar calendar = Calendar.getInstance();
            calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
            queryWrapper.ge("create_date", calendar.getTime());
            List<EnergyElectricityDataVo> energyElectricityDataVos = dataMapper.listDay4(queryWrapper);
            electricityList = energyElectricityDataVos;
        }
        return new Result(ResultCode.SUCCESS, electricityList);
    }
}
